//********   2DLib del Simulador  ***********
#include "2Dlib.h"

/**
*****************************************************************************
* Reserva espacio en memoria para una matriz de nxn elementos
* @param N Orden de la matriz
* @return Puntero a la matriz a alocatada
* @see FreeMatrix()
****************************************************************************/
matriz IniMatrix (int n)
{
	matriz matrix;
	int i;

	matrix=(double**)malloc(n*sizeof(double*));
	for (i=0;i<n;i++){
		matrix[i]=(double*)calloc(n,sizeof(double));
	}
	return matrix;
}

/**
*****************************************************************************
* Libera el espacio reservado en memoria de una matriz de nxn elementos
* @param Matrix Puntero a la matriz que queremos desalocatar
* @param n Orden de la matriz que queremos desalocatar
* @see IniMatrix
****************************************************************************/
void FreeMatrix (matriz matrix,int n)
{
	int i;
	for (i=0;i<n;i++) free (matrix[i]);
	free (matrix);
}
	
/**
*****************************************************************************
* Multiplica dos matrices de 3x3 elementos
* @param a Puntero a la primera matriz que queremos multiplicar
* @param b Puntero a la segunda matriz que queremos multiplicar
* @param c Devuelve un puntero a la matriz resultante de multiplicar axb
****************************************************************************/
void mulmat (matriz a, matriz b, matriz c)
{
	int i,j,k;

	for (i=0;i<3;i++){
		for (j=0;j<3;j++){
			c[i][j]=0.0;
			for(k=0;k<3;k++){
				c[i][j]=c[i][j]+(a[i][k]*b[k][j]);
			}
		}
	}
}

/**
*****************************************************************************
* Determina los valores de una matriz de giro de 3x3 elementos
* @param Matrix Matriz en la que se almacenarn los valores de la matriz de giro
* @param Angulo Angulo en grados del giro que queremos realizar
* @see IniGiroRespectoPunto()
****************************************************************************/
void IniGiro (matriz matrix, double Angulo)
{						   
	Angulo=(Angulo*PI)/180;
	matrix[0][0]=matrix[1][1]=cos(Angulo);
	matrix[0][1]=sin(Angulo);
	matrix[1][0]=-sin(Angulo);
	matrix[2][0]=matrix[2][1]=0;
	matrix[2][2]=1;
}

/**
*****************************************************************************
* Determina los valores de una matriz de giro de 3x3 elementos, respecto a 
un punto dado 
* @param Matrix Matriz en la que se almacenarn los valores de la matriz de giro
* @param Angulo ngulo en grados del giro que queremos realizar
* @param x Coordenada X del punto respecto al que se realiza el giro
* @param y Coordenada Y del punto respecto al que se realiza el giro
* @see IniGiro()
****************************************************************************/
void IniGiroRespectoPunto (matriz matrix, double Angulo, double x,double y)
{
	double AnguloRAD;

	AnguloRAD=(Angulo*PI)/180.0;
	IniGiro(matrix,Angulo);
	matrix[2][0]=-x*(1-cos(AnguloRAD))+y*sin(AnguloRAD);
	matrix[2][1]=y*(1-cos(AnguloRAD))+x*sin(AnguloRAD);
}

/**
*****************************************************************************
* Determina los valores de una matriz de traslacin de 3x3 elementos
* @param Matrix Matriz en la que se almacenarn los valores de la matriz de traslacin
* @param x Coordenada X del punto al que queremos deplazarnos
* @param y Coordenada Y del punto al que queremos deplazarnos
****************************************************************************/
void IniTraslacion (matriz matrix, double x, double y)
{
	matrix[0][0]=matrix[1][1]=matrix[2][2]=1;
	matrix[2][0]=long(x);
	matrix[2][1]=long(y);
}

/**
*****************************************************************************
* Determina los valores de una matriz de simetra respecto al eje X, de 3x3 elementos
* @param Matrix Matriz en la que se almacenarn los valores de la matriz de simetra 
* @see IniSimetriaY()
****************************************************************************/
void IniSimetriaX (matriz matrix)
{
	matrix[0][0]=matrix[2][2]=1;
	matrix[1][1]=-1;
}

/**
*****************************************************************************
* Determina los valores de una matriz de simetra respecto al eje Y, de 3x3 elementos
* @param Matrix Matriz en la que se almacenarn los valores de la matriz de simetra 
* @see IniSimetriaX()
****************************************************************************/
void IniSimetriaY (matriz matrix)
{
	matrix[1][1]=matrix[2][2]=1;
	matrix[0][0]=-1;
}

/**
*****************************************************************************
* Determina los valores de una matriz de escalado de 3x3 elementos
* @param Matrix Matriz en la que se almacenarn los valores de la matriz de escalado
* @param EscaladoX Escala que queremos en el eje X
* @param EscaladoY Escala que queremos en el eje Y
****************************************************************************/
void IniEscalado (double EscaladoX,double EscaladoY,matriz matrix)
{
	matrix[0][0]=EscaladoX;
	matrix[1][1]=EscaladoY;
	matrix[2][2]=1;
}

/**
*****************************************************************************
* 
****************************************************************************/
void IniEscaladoRespectoPunto (double EscaladoX,double EscaladoY,
							   double X,double Y,matriz matrix)
{
	IniEscalado (EscaladoX,EscaladoY,matrix);
	matrix[2][0]=X*(1-EscaladoX);
	matrix[2][1]=-Y*(1+EscaladoY);
}

/**
*****************************************************************************
* Multiplica un punto 2D por una matriz 3x3 (asumiendo que la tercera componente
del vector, necesaria para la multiplicacin, es 1)
* @param Matrix Matriz por la que queremos multiplicar el vector
* @param Punto Punto 2D al que queremos aplicar la transformacin de la matriz
* @return Punto 2D resultante de la transformacin
****************************************************************************/
Pt2D MultVect (matriz matrix, Pt2D punto)
{
	Pt2D p;

	p.x=int((punto.x*matrix[0][0])+(punto.y*matrix[1][0])+(matrix[2][0]));
	p.y=int((punto.x*matrix[0][1])+(punto.y*matrix[1][1])+(matrix[2][1]));
	return p;
}

/**
*****************************************************************************
* Calcula la inversa de una matriz 3x3
* @param Matrix Puntero a la matriz de la que queremos realizar la inversa
* @return Puntero a la matriz inversa
****************************************************************************/
matriz Inversa3X3 (matriz matrix)
{
	double tmp;
	matriz resultado;

	resultado=IniMatrix(3);

	tmp=+(matrix[0][0]*matrix[1][1]*matrix[2][2])+(matrix[1][0]*matrix[0][2]*matrix[2][1])+(matrix[2][0]*matrix[0][1]*matrix[1][2])
		-(matrix[0][0]*matrix[1][2]*matrix[2][1])-(matrix[1][0]*matrix[0][1]*matrix[2][2])-(matrix[2][0]*matrix[0][2]*matrix[1][1]);

	resultado[0][0]=((matrix[1][1]*matrix[2][2])-(matrix[1][2]*matrix[2][1]))/tmp;
	resultado[0][1]=-(((matrix[0][1]*matrix[2][2])-(matrix[0][2]*matrix[2][1]))/tmp);
	resultado[0][2]=((matrix[0][1]*matrix[1][2])-(matrix[0][2]*matrix[1][1]))/tmp;
	resultado[1][0]=-(((matrix[1][0]*matrix[2][2])-(matrix[1][2]*matrix[2][0]))/tmp);		
	resultado[1][1]=((matrix[0][0]*matrix[2][2])-(matrix[0][2]*matrix[2][0]))/tmp;
	resultado[1][2]=-(((matrix[0][0]*matrix[1][2])-(matrix[0][2]*matrix[1][0]))/tmp);		
	resultado[2][0]=((matrix[1][0]*matrix[2][1])-(matrix[1][1]*matrix[2][0]))/tmp;
	resultado[2][1]=-(((matrix[0][0]*matrix[2][1])-(matrix[0][1]*matrix[2][0]))/tmp);
	resultado[2][2]=((matrix[0][0]*matrix[1][1])-(matrix[0][1]*matrix[1][0]))/tmp;
	return resultado;
}
